@@ -3,11 +3,11 @@
 
 To simplify the management of function default keyword argument values
 :py:mod:`funconf` introduces two new decorators. The first decorator 
-:py:func:`wraps_kwargs` makes it trivial to dynamically define the default
-kwargs for a function.  The second decorator :py:func:`lazy_string_cast`
-automatically casts *basestring* values to the type of the keyword default
-values found in the function it is wrapping and the type of the values found in
-the *key:value* object passed into its constructor. 
+:py:func:`wraps_parameters` makes it trivial to dynamically define the
+default values of parameters to a function.  The second decorator
+:py:func:`lazy_string_cast` automatically casts *basestring* values to the type
+of the keyword default values found in the function it is wrapping and the type
+of the values found in the *key:value* object passed into its constructor. 
 
 For configuration, :py:mod:`funconf` borrows from concepts discussed in
 Python's core library *ConfigParser*.  A configuration consists of sections
@@ -62,7 +62,7 @@ A configuration object
 
 The :py:class:`Config` class is the root class container for all of the
 configuration sections.  It implements the *MutableMapping* type which allows
-it to seamlessly integrate into the :py:func:`wraps_kwargs` function. 
+it to seamlessly integrate into the :py:func:`wraps_parameters` function. 
 
 As a dictionary ``dict(config)`` the :py:class:`Config` object looks like::
 
@@ -92,7 +92,7 @@ To access sections and option values the following pattern holds true::
 
 A section is represented by the :py:class:`ConfigSection` object. This object
 implements the *MutableMapping* type in the same way the :py:class:`Config`
-object is implemented and is compatible with the :py:func:`wraps_kwargs`
+object is implemented and is compatible with the :py:func:`wraps_parameters`
 function.  The :py:class:`ConfigSection` represented ``str(config.bread)``
 looks like::
 
@@ -109,8 +109,12 @@ used as a decorator to a function too. Here is an example::
 
 """
 import functools
-import inspect
+from inspect import isfunction, ismethod 
 from collections import MutableMapping
+try:
+    from collections import OrderedDict 
+except ImportError:
+    from ordereddict import OrderedDict
 import shlex
 from distutils.util import strtobool
 try:
@@ -124,89 +128,135 @@ except NameError:
 import yaml
 
 
-def wraps_kwargs(default_kwargs):
-    """Decorate a function to define and extend its default keyword argument
-    values.
+def wraps_parameters(default_kwargs, hide_var_keyword=False,
+                                     hide_var_positional=False):
+    """Decorate a function to define and extend its positional and keyword
+    variables.
         
-    The following example will redefine myfunc to have default kwargs of a=4
-    and b=2:: 
+    The following example will redefine myfunc to have defaults of a=4 and b=3::
 
-        mydict = {a=4, b=2}
-        @wraps_kwargs(mydict)
-        def myfunc(a=2, **k):
+        mydict = {a=4, b=3}
+        @wraps_parameters(mydict)
+        def myfunc(a, b=2):
             pass
 
     The *default_kwargs* object needs to satisfy the *MutableMapping* interface
     definition. When a wrapped function is called the following transforms
     occur over *kwargs* before it is passed into the wrapped function:
 
-        1. *default_kwargs* is updated with keywords found in the *kwargs*.
-        2. The functions default keyword arguments that were not in *kwargs*
-           and are defined by *default_kwargs* will be copied into *kwargs*
-           from the *default_kwargs* object.
-        3. If the wrapped function has a variable keyword argument defined 
+        1. *default_kwargs* is updated with new input parameters.
+        2. If the wrapped function has a variable keyword argument defined 
            (i.e ``**k``) then the keywords defined by *default_kwargs* that are
            not defined in *kwargs* will be copied into *kwargs*.
-        4. If the wrapped function had no variable keyword argument defined,
-           then the values found in *kwargs* that are no in the wrapped
-           function's keyword argument list  will be popped from *kwargs*.
+        3. If the wrapped function had no variable keyword argument defined,
+           then the keyword input parameters that don't belong to the wrapped
+           function's parameters list will be discarded.
 
     :param default_kwargs: kwargs to be fix into the wrapped function.
     :type default_kwargs: mutable mapping
+    :param hide_var_keyword: hide the variable keyword parameter.
+    :type hide_var_keyword: Boolean value default True.
+    :param hide_var_arguments: hide the variable keyword parameter.
+    :type hide_var_arguments: Boolean value default True.
     :rtype: decorated function.
     """
     def decorator(func):
-        default_kwargs_set = set(default_kwargs)
-        func_defaults_set = set()
-        override_defaults_set = set()
-        has_var_keyword = False
-        def wrapper(**kwargs):
-            kwargs_set = set(kwargs)
-            # Update default_kwargs with new keyword values.
-            for k in kwargs_set.intersection(default_kwargs_set):
-                default_kwargs[k] = kwargs[k]
-            # Override func's default keyword values.
-            for k in override_defaults_set.difference(kwargs_set):
-                kwargs[k] = default_kwargs[k]
-            if has_var_keyword:
+        # Build new signature.
+        original_sig = signature(func)
+        var_keyword = '' 
+        var_positional = '' 
+        parameters = OrderedDict()
+        original_positional = OrderedDict()
+        # Add positional arguments and keywords first.
+        for name, param in original_sig.parameters.items():
+            if param.kind == param.VAR_KEYWORD:
+                var_keyword = name
+            elif param.kind == param.VAR_POSITIONAL:
+                var_positional = name
+            else:
+                default = default_kwargs.get(name, param.default)
+                param = Parameter(name, param.kind, default=default)
+                parameters[name] = param
+                original_positional[name] = param
+        # Add var positional.
+        if var_positional:
+            parameters[var_positional] = Parameter(var_positional, 
+                                                   Parameter.VAR_POSITIONAL)
+        # Add remainder defualt_kwargs as keyword only variables.
+        for name, value in default_kwargs.items():
+            if name not in parameters:
+                param = Parameter(name, Parameter.KEYWORD_ONLY, default=value)
+                parameters[name] = param
+        # Add var keyword.
+        if var_keyword:
+            parameters[var_keyword] = Parameter(var_keyword,
+                                                Parameter.VAR_KEYWORD)
+        # Build our inner wrapper signature.
+        wrapper_sig = original_sig.replace(parameters=parameters.values())
+        # Remove cloaked var arguments.
+        if var_positional and hide_var_positional:
+            parameters.pop(var_positional)
+        if var_keyword and hide_var_keyword:
+            parameters.pop(var_keyword)
+        cloak_sig = original_sig.replace(parameters=parameters.values())
+
+        # Wrapper function.
+        function_defaults = set(original_sig.parameters)
+        override_defaults = set(default_kwargs).intersection(function_defaults)
+        def wrapper(*args, **kwargs):
+            # Build new kwargs and args.
+            arguments = OrderedDict(wrapper_sig.bind(*args, **kwargs).arguments)
+            kwargs = {}
+            updates = {}
+            
+            # Build the positional arguments. Override func's default values.
+            ordered_args = OrderedDict()
+            for name in original_positional:
+                if name in arguments:
+                    ordered_args[name] = arguments[name]
+                    if name in default_kwargs:
+                        updates[name] = arguments[name]
+                elif name in default_kwargs:
+                    ordered_args[name] = default_kwargs[name]
+                    updates[name] = default_kwargs[name]
+            
+            # Now handle the keyword only and var arguments
+            args = list(ordered_args.values())
+            for name in set(arguments).difference(ordered_args):
+                value = arguments[name]
+                if name == var_positional:
+                    args.extend(value)
+                elif name == var_keyword: 
+                    for k, v in value.items():
+                        kwargs[k] = v
+                else:
+                    # Update keyword only values
+                    if name in default_kwargs:
+                        updates[name] = value
+                    kwargs[name] = value
+
+            default_kwargs.update(updates)
+            if var_keyword:
                 # Add default_kwargs keyword values not defined in kwargs.
-                for k in default_kwargs_set.difference(kwargs_set):
-                    kwargs[k] = default_kwargs[k]
+                for k in set(default_kwargs).difference(kwargs):
+                    if k not in original_positional:
+                        kwargs[k] = default_kwargs[k]
             else:
                 # Remove kwargs that func doesn't have defined.
-                for k in kwargs_set.difference(func_defaults_set):
+                for k in set(kwargs).difference(function_defaults):
                     kwargs.pop(k)
-            return func(**kwargs)
+            return func(*args, **kwargs)
 
-        funcsig = signature(func)
-        parameters = []
-        for arg, param in funcsig.parameters.items():
-            if param.default != param.empty:
-                func_defaults_set.add(arg)
-                if arg in default_kwargs_set:
-                    override_defaults_set.add(arg)
-                else:
-                    param = Parameter(arg, Parameter.KEYWORD_ONLY,
-                                           default=param.default)
-                    parameters.append(param)
-            elif param.kind == param.VAR_KEYWORD:
-                has_var_keyword = True
-            else:
-                msg = "%s parameters are not supported" % param.kind 
-                raise ValueError(msg)
+        # Return wrapped up func with the cloaked signature. 
         functools.update_wrapper(wrapper, func)
-        for k, v in default_kwargs.items():
-            param = Parameter(k, Parameter.KEYWORD_ONLY, default=v)
-            parameters.append(param)
-        sig = Signature(parameters=parameters)
-        wrapper.__signature__ = sig
+        wrapper.__signature__ = cloak_sig
         return wrapper
     return decorator
 
 
-def lazy_string_cast(model_kwargs={}):
+def lazy_string_cast(model_parameters={}):
     """Type cast string input values if they differ from the type of the
-    default value found in *model_kwargs*.
+    default value found in *model_parameters*.
     
     The following list details how each type is handled:
 
@@ -231,18 +281,18 @@ def lazy_string_cast(model_kwargs={}):
             pass
 
 
-    Or using :py:func:`lazy_string_cast` with :py:func:`wraps_kwargs` to
+    Or using :py:func:`lazy_string_cast` with :py:func:`wraps_parameters` to
     define new keyword defaults::
 
         config = dict(a=6, b=[4])
 
         @lazy_string_cast(config)
-        @wraps_kwargs()
+        @wraps_parameters()
         def main(a=4, b=[4, 2, 55]):
             pass
 
-    :param model_kwargs: kwargs to model default type values and keys from.
-    :type model_kwargs: mutable mapping
+    :param model_parameters: kwargs to model default type values and keys from.
+    :type model_parameters: mutable mapping
     :rtype: decorated function.
     """
     def cast_type_raise(vtype, key, value):
@@ -282,27 +332,58 @@ def lazy_string_cast(model_kwargs={}):
         else:
             return make_cast_func(cast_type, k, vtype)
 
+    class StrCast(dict):
+        def __call__(self, name, value):
+            if isinstance(value, basestring) and name in self:
+                return self[name](value)
+            else:
+                return value
+
     def decorator(func):
-        cast_ctrl = {}
-        for k, v in model_kwargs.items():
-            cast_ctrl[k] = (type(v), cast_factory(k, v))
-        funcsig = signature(func)
-        for k, param in funcsig.parameters.items():
-            if param.default != param.empty:
-                v = param.default
-                cast_ctrl[k] = (type(v), cast_factory(k, v))
+        sig = signature(func)
+        var_keyword, var_positional = '', '' 
+        positional = []
+        # Build the StrCast object
+        str_cast = StrCast() 
+        for name, param in sig.parameters.items():
+            if param.default != param.empty and \
+                    not isinstance(param.default, basestring):
+                str_cast[name] = cast_factory(name, param.default)
+            if param.kind == param.VAR_KEYWORD:
+                var_keyword = name
+            elif param.kind == param.VAR_POSITIONAL:
+                var_positional = name
+            elif param.kind == param.POSITIONAL_OR_KEYWORD:
+                positional.append(name)
+        for name, value in model_parameters.items():
+            if not isinstance(value, basestring):
+                str_cast[name] = cast_factory(name, value)
+
         @functools.wraps(func)
-        def wrapper(**kwargs):
-            for k, v in kwargs.items():
-                if k in cast_ctrl:
-                    vtype, cast_func = cast_ctrl[k]
-                    if not isinstance(v, vtype) and isinstance(v, basestring): 
-                        kwargs[k] = cast_func(v)
-            return func(**kwargs)
+        def wrapper(*args, **kwargs):
+            arguments = OrderedDict(sig.bind(*args, **kwargs).arguments)
+            # Cast the function's positional arguments.
+            ordered_args = OrderedDict()
+            for name in positional:
+                ordered_args[name] = str_cast(name, arguments[name])
+            args = list(ordered_args.values())
+            # Cast the function's keyword arguments.
+            kwargs = {}
+            for name in set(arguments).difference(ordered_args):
+                value = arguments[name]
+                if name == var_positional:
+                    args.extend(value)
+                elif name == var_keyword: 
+                    for k, v in value.items():
+                        kwargs[k] = str_cast(k, v)
+                else:
+                    kwargs[name] = str_cast(name, value)
+            return func(*args, **kwargs)
+        wrapper.__signature__ = sig
         return wrapper
 
-    if inspect.isfunction(model_kwargs) or inspect.ismethod(model_kwargs):
-        func, model_kwargs = model_kwargs, {}
+    if isfunction(model_parameters) or ismethod(model_parameters):
+        func, model_parameters= model_parameters, {}
         return decorator(func)
     else:
         return decorator
@@ -320,7 +401,7 @@ class ConfigSection(MutableMapping):
         1. Exposes a configuration *option:value* through a standard
            implementation of the *MutableMapping* abstract type.
         2. When cast to a string it outputs YAML.
-        3. As a decorator it utilises the :py:func:`wraps_kwargs` to change
+        3. As a decorator it utilises the :py:func:`wraps_parameters` to change
            the defaults of a variable kwargs function.  
     """
  
@@ -410,7 +491,7 @@ class ConfigSection(MutableMapping):
         self._dirty, d = False, self._dirty
         return d
 
-    def __call__(self, func=None, **kwargs):
+    def __call__(self, func=None, lazy=True):
         """The :py:class:`ConfigSection` object can be used as a function
         decorator.  
 
@@ -436,16 +517,14 @@ class ConfigSection(MutableMapping):
         :rtype: As a factory returns decorator function. As a decorator
                 function returns a decorated function. 
         """
-        lazy = kwargs.get('lazy', True)
-        def _wraps(func):
-            if lazy:
-                return lazy_string_cast(self)(wraps_kwargs(self)(func)) 
-            else:
-                return wraps_kwargs(self)(func)
-        if inspect.isfunction(func) or inspect.ismethod(func):
-            return _wraps(func)
+        if func is None:
+            return functools.partial(self, lazy=lazy)
+        wrapped = wraps_parameters(self, hide_var_positional=True,
+                                     hide_var_keyword=True)(func)
+        if lazy:
+            return lazy_string_cast(self)(wrapped) 
         else:
-            return _wraps
+            return wrapped
 
 
 ConfigSection._reserved = set(dir(ConfigSection))
@@ -463,7 +542,7 @@ class Config(MutableMapping):
            through a standard implementation of the *MutableMapping* abstract
            type.
         3. When cast to a string it outputs YAML.
-        4. As a decorator it utilises the :py:func:`wraps_kwargs` to change the
+        4. As a decorator it utilises the :py:func:`wraps_parameters` to change the
            defaults of a variable kwargs function.  
     """
 
@@ -614,7 +693,7 @@ class Config(MutableMapping):
         section = self._sections[s]
         return section[option]
 
-    def __call__(self, func=None, **kwargs):
+    def __call__(self, func=None, lazy=True):
         """The :py:class:`Config` object can be used as a function decorator.  
         
         Applying this decorator to a function which takes variable kwargs will
@@ -639,16 +718,15 @@ class Config(MutableMapping):
         :rtype: As a factory returns decorator function. As a decorator
                 function returns a decorated function. 
         """
-        lazy = kwargs.get('lazy', True)
-        def _wraps(func):
-            if lazy:
-                return lazy_string_cast(self)(wraps_kwargs(self)(func)) 
-            else:
-                return wraps_kwargs(self)(func)
-        if inspect.isfunction(func) or inspect.ismethod(func):
-            return _wraps(func)
+        if func is None:
+            return functools.partial(self, lazy=lazy)
+        wrapped = wraps_parameters(self, hide_var_positional=True,
+                                     hide_var_keyword=True)(func)
+        if lazy:
+            return lazy_string_cast(self)(wrapped) 
         else:
-            return _wraps
+            return wrapped
+
 
 Config._reserved = set(dir(Config))
      
